# Copyright 2020 The ASReview Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import matplotlib.pyplot as plt
import numpy as np

from asreview.analysis.analysis import Analysis


def _add_WSS(WSS, analysis, ax, col, result_format, box_dist=0.5, **kwargs):
    if WSS is None:
        return

    text = f"WSS@{WSS}%"
    _, WSS_x, WSS_y = analysis.wss(WSS, x_format=result_format, **kwargs)
    if WSS_x is None or WSS_y is None:
        return

    text_pos_x = WSS_x[0] + box_dist
    text_pos_y = (WSS_y[0] + WSS_y[1])/2
    plt.plot(WSS_x, WSS_y, color=col)
    bbox = dict(boxstyle='round', facecolor=col, alpha=0.5)
    ax.text(text_pos_x, text_pos_y, text, color="white", bbox=bbox)


def _add_RRF(RRF, analysis, ax, col, result_format, box_dist=0.5, **kwargs):
    if RRF is None:
        return

    text = f"RRF@{RRF}%"
    _, RRF_x, RRF_y = analysis.rrf(RRF, x_format=result_format, **kwargs)
    if RRF_x is None or RRF_y is None:
        return

    text_pos_x = RRF_x[0] + box_dist
    text_pos_y = (RRF_y[0] + RRF_y[1])/2
    plt.plot(RRF_x, RRF_y, color=col)
    bbox = dict(boxstyle='round', facecolor=col, alpha=0.5)
    ax.text(text_pos_x, text_pos_y, text, color="white", bbox=bbox)


class Plot():
    def __init__(self, data_dirs):
        self.analyses = {}

        for data_dir in data_dirs:
            new_analysis = Analysis.from_dir(data_dir)
            if new_analysis is not None:
                data_key = new_analysis.key
                self.analyses[data_key] = new_analysis

    def __enter__(self):
        return self

    def __exit__(self, *_, **__):
        for analysis in self.analyses.values():
            analysis.close()

    @classmethod
    def from_dirs(cls, data_dirs):
        plot_inst = Plot(data_dirs)
        if len(plot_inst.analyses) == 0:
            return None
        return plot_inst

    def plot_time_to_inclusion(self, X_fp):
        for data_key, analysis in self.analyses.items():
            results = analysis.time_to_inclusion(X_fp)
            for key in results["ttd"]:
                plt.plot(results["x_range"], results["ttd"][key],
                         label=data_key + " - " + key)
        plt.legend()
        plt.show()

    def plot_time_to_discovery(self, result_format="percentage"):
        avg_times = []
        for analysis in self.analyses.values():
            results = analysis.avg_time_to_discovery(
                result_format=result_format)
            avg_times.append(list(results.values()))

        if result_format == "number":
            plt.hist(avg_times, 30, histtype='bar', density=False,
                     label=self.analyses.keys())
            plt.xlabel("# Reviewed")
            plt.ylabel("# of papers included")
        else:
            plt.hist(avg_times, 30, histtype='bar', density=True,
                     label=self.analyses.keys())
            plt.xlabel("% Reviewed")
            plt.ylabel("Fraction of papers included")
        plt.legend()
        plt.show()

    def plot_inc_found(self, result_format="percentage", abstract_only=False):
        """
        Plot the number of queries that turned out to be included
        in the final review.
        """
        legend_name = []
        legend_plt = []

        fig, ax = plt.subplots()

        max_len = 0
        for i, data_key in enumerate(self.analyses):
            analysis = self.analyses[data_key]

            inc_found = analysis.inclusions_found(result_format=result_format)
            n_after_init = len(analysis.labels) - analysis.inc_found[False]["n_initial"]
            max_len = max(max_len, n_after_init)
            if result_format == "percentage":
                box_dist = 0.5
            else:
                box_dist = 100
            col = "C"+str(i % 10)
            _add_WSS(95, analysis, ax, col, result_format, box_dist)
            _add_WSS(100, analysis, ax, col, result_format, box_dist)
            _add_RRF(10, analysis, ax, col, result_format, box_dist)
            _add_RRF(5, analysis, ax, col, result_format, box_dist)

            myplot = plt.errorbar(*inc_found, color=col)
            if abstract_only:
                legend_name.append(f"{data_key} (abstract)")
            else:
                legend_name.append(f"{data_key}")
            legend_plt.append(myplot)

            if abstract_only:
                col = "red"
                inc_found_final = analysis.inclusions_found(
                    result_format=result_format, final_labels=True)
                _, WSS95_x, _ = analysis.wss(95, x_format=result_format, final_labels=True)
                _, WSS100_x, _ = analysis.wss(100, x_format=result_format, final_labels=True)
                bbox = dict(boxstyle='round', facecolor=col, alpha=0.5)
                prev_value = 0
                x_vals = []
                y_vals = []
                WSS_added = False
                for i in range(len(inc_found_final[0])):
                    if inc_found_final[1][i] != prev_value:
                        x_vals.append(inc_found_final[0][i])
                        y_vals.append(inc_found[1][i])
                        prev_value = inc_found_final[1][i]
                        if inc_found_final[0][i] >= WSS100_x[0] - 1e-4 and not WSS_added:
                            ax.text(WSS100_x[0]+300, inc_found[1][i], "WSS@100%", color="white", bbox=bbox)
                            WSS_added = True
                myplot = plt.scatter(x_vals, y_vals, color=col)
                legend_name.append(f"{data_key} (final)")
                legend_plt.append(myplot)

        plt.legend(legend_plt, legend_name, loc="lower right")

        if result_format == "number":
            ax2 = ax.twiny()
            ax.set_xlim(0, max_len)
            ax2.set_xlim(0, 100)
            ax.set_xlabel("# Reviewed")
            ax2.set_xlabel("% Reviewed")
            ax.set_ylabel("# Inclusions found")
            symb = "#"
        elif result_format == "percentage":
            symb = "%"
            ax.set_xlabel("% Reviewed")
            ax.set_ylabel("% Inclusions found")
        else:
            symb = "?"

        plt.grid()
        fig.tight_layout()
        plt.show()

    def plot_limits(self, prob_allow_miss=[0.1, 0.5, 2.0],
                    result_format="percentage"):
        legend_plt = []
        legend_name = []
        linestyles = ['-', '--', '-.', ':']

        for i, data_key in enumerate(self.analyses):
            res = self.analyses[data_key].limits(
                prob_allow_miss=prob_allow_miss,
                result_format=result_format)
            x_range = res["x_range"]
            col = "C"+str(i % 10)

            for i_limit, limit in enumerate(res["limits"]):
                ls = linestyles[i_limit % len(linestyles)]
                my_plot, = plt.plot(x_range, np.array(limit)+np.array(x_range),
                                    color=col, ls=ls)
                if i_limit == 0:
                    legend_plt.append(my_plot)
                    legend_name.append(f"{data_key}")

        plt.plot(x_range, x_range, color="black", ls='--')
        if result_format == "percentage":
            plt.xlabel("% of papers read")
            plt.ylabel("Estimate of % of papers that need to be read")
        else:
            plt.xlabel("# of papers read")
            plt.ylabel("Estimate of # of papers that need to be read")
        plt.legend(legend_plt, legend_name, loc="upper right")
        plt.title("Articles left to read")
        plt.grid()
        plt.show()



# based on https://github.com/pypa/sampleproject
# MIT License

# Always prefer setuptools over distutils
from setuptools import setup, find_namespace_packages
from os import path
from io import open

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

# Extract version from cbsodata.py
for line in open(path.join("asreviewcontrib", "visualization", "__init__.py")):
    if line.startswith('__version__'):
        exec(line)
        break

setup(
    name='asreview-visualization',
    version=__version__,  # noqa
    description='Visualization tools for the ASReview project',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/msdslab/ASReview-visualization',
    author='Utrecht University',
    author_email='asreview@uu.nl',
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Pick your license as you wish
        'License :: OSI Approved :: Apache Software License',

        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    keywords='asreview plot visualization',
    packages=find_namespace_packages(include=['asreviewcontrib.*']),
    install_requires=[
        "numpy", "matplotlib", "asreview",
    ],

    extras_require={
    },

    entry_points={
        "asreview.entry_points": [
            "plot = asreviewcontrib.visualization.entrypoint:PlotEntryPoint",
        ]
    },

    project_urls={
        'Bug Reports':
            "https://github.com/msdslab/ASReview-visualization/issues",
        'Source':
            "https://github.com/msdslab/ASReview-visualization",
    },
)



# Copyright 2020 The ASReview Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse

from asreview.entry_points import BaseEntryPoint

from asreviewcontrib.visualization import Plot
import logging

PLOT_TYPES = ['inclusions', 'discovery', 'limits']


class PlotEntryPoint(BaseEntryPoint):
    description = "Plotting functionality for logging files produced by "\
        "ASReview."
    extension_name = "asreview-visualization"

    def __init__(self):
        from asreviewcontrib.visualization.__init__ import __version__
        super(PlotEntryPoint, self).__init__()

        self.version = __version__

    def execute(self, argv):
        parser = _parse_arguments()
        args_dict = vars(parser.parse_args(argv))

        if args_dict['type'] == 'all':
            types = PLOT_TYPES
        else:
            arg_types = args_dict['type'].split(',')
            types = []
            for plot_type in arg_types:
                if plot_type not in PLOT_TYPES:
                    logging.warning(f'Plotting type "{plot_type} unknown."')
                else:
                    types.append(plot_type)

        if args_dict["absolute_format"]:
            result_format = "number"
        else:
            result_format = "percentage"

        with Plot.from_dirs(args_dict["data_dirs"]) as plot:
            if "inclusions" in types:
                plot.plot_inc_found(result_format=result_format)
            if "discovery" in types:
                plot.plot_time_to_discovery(result_format=result_format)
            if "limits" in types:
                plot.plot_limits(result_format=result_format)


def _parse_arguments():
    parser = argparse.ArgumentParser(prog='asreview plot')
    parser.add_argument(
        'data_dirs',
        metavar='N',
        type=str,
        nargs='+',
        help='Data directories.'
    )
    parser.add_argument(
        "-t", "--type",
        type=str,
        default="all",
        help="Type of plot to make. Separate by commas (no spaces) for"
        " multiple plots."
    )
    parser.add_argument(
        "-a", "--absolute-values",
        dest="absolute_format",
        action='store_true',
        help='Use absolute values on the axis instead of percentages.'
    )
    return parser
